#include <Eventually.h>
#include "Adafruit_TLC59711.h"
#include <SD.h>
#include <SPI.h>
#include <Tsunami.h> 

const int BUTTON_PIN[10] = {30, 31, 32, 33, 34, 35, 36, 37, 38, 39} ;
int currentButton ;

// TLC59711
#define NUM_TLC59711 1
#define data   28
#define clock  27
Adafruit_TLC59711 tlc = Adafruit_TLC59711(NUM_TLC59711, clock, data);
uint32_t ledState[12] ;

// SD card
File btnFile ;
String buf;
boolean SDfound;

//Tsunami wav player
Tsunami tsunami;
int  gNumTracks;                // Number of tracks on SD card
char gTsunamiVersion[VERSION_STRING_LEN];    // Tsunami version string

//Eventually pointers to manage creation/deletion of listeners
EvtManager mgr;



bool state = LOW;
int test;
int test2;

EvtListener *plist;
EvtListener *plist2;

//Functions prototypes
void EvtResetButtonContext ();
void tlcWrite();
void buttonFocus(int);
//////////////////////// SETUP /////////////////////////////////

void setup() {
  Serial.begin(9600);

  // TLC
  tlc.begin();
  tlc.write();

   for (int i ; i<12; i++) {
      Serial.println(i);
      ledState[i] = 65535 ;
      tlcWrite();
      delay (100);
      ledState[i] = 0 ;
      tlcWrite();
   }  

   // SD
   if (SDfound == 0) {
    if (!SD.begin(BUILTIN_SDCARD)) {
      Serial.print("The SD card cannot be found");
      while(1);
    }
  }
  SDfound = 1;

  // TSUNAMI
  // We should wait for the Tsunami to finish reset 
  delay(1000);
  tsunami.start();
  delay(10);
  tsunami.stopAllTracks();
  tsunami.samplerateOffset(0, 0);
  tsunami.setReporting(true);
  delay(100); 
  if (tsunami.getVersion(gTsunamiVersion, VERSION_STRING_LEN)) {
    Serial.print(gTsunamiVersion);
    Serial.print("\n");
    gNumTracks = tsunami.getNumTracks();
    Serial.print("Number of tracks = ");
    Serial.print(gNumTracks);
    Serial.print("\n");
  }
  else
    Serial.print("WAV Trigger response not available");

  //Set button pins as input
  for (int i ; i < sizeof(BUTTON_PIN)/sizeof(int) ; i++) {
    pinMode(BUTTON_PIN[i],INPUT_PULLUP) ; 
  }

  //EvtResetButtonContext();
  buttonFocus(0) ;
  
//  plist = new EvtTimeListener(1000, true, (EvtAction)blinkme);
//  plist2 = new EvtTimeListener(100, true, (EvtAction)blinkme2);
//  mgr.addListener( plist ); 
//  //mgr.addListener( plist2 ); 
}


///////////////////////// SD ////////////////////////////////////////////

String getSubstring(String dataString, char separator, int index)
{
    int found = 0;
    int strIndex[] = { 0, -1 };
    int maxIndex = dataString.length() - 1;

    for (int i = 0; i <= maxIndex && found <= index; i++) {
        if (dataString.charAt(i) == separator || i == maxIndex) {
            found++;
            strIndex[0] = strIndex[1] + 1;
            strIndex[1] = (i == maxIndex) ? i+1 : i;
        }
    }
    return found > index ? dataString.substring(strIndex[0], strIndex[1]) : "";
}



void readSDFile() {
 
  while (btnFile.available()) {
    buf = btnFile.readStringUntil('\n');
    //check selector (1st word of line) and execute corresponding function
    String selector = getSubstring(buf, ' ', 0);
    Serial.println(selector);
    if (selector == "sound") {
      String track = getSubstring(buf, ' ', 2) ;
      String out = getSubstring(buf, ' ', 1) ;
      Serial.println("Playing track " + track + "to output " + out);
      tsunami.trackPlayPoly(track.toInt(), out.toInt(), true);
      
    }
    else if (selector == "led") {
      Serial.println("Led mode " + getSubstring(buf, ' ', 1));
    }
    else if (selector == "dimmer") {
      Serial.println("Dimmer fade à " + getSubstring(buf, ' ', 1) + " en " + getSubstring(buf, ' ', 2) + "ms");
    }
    else if (selector == "delay") {
      String wait = getSubstring(buf, ' ', 1);
      Serial.println("Waiting " + wait + " ms");
      delay(wait.toInt());
    }
    else {Serial.println("pas compris");}
    
    
  }
  Serial.println("EOF");
  btnFile.close();
  
}

/////////////////////////////Events/////////////////////////////////
///*resetButtonContext create a listener for each button
// * when triggered, thoses listener execute the same functions with the index of the pressed button  :
// * resetButtonContext and kill any running sequence 
// * buttonFocus -> lights the current button and shuts the others
// * readSequence -> reads a text file on the SD card with the sequence of actions to execute
// */
//
//
//
void tlcWrite(){
  for (int ledIndex ; ledIndex < 12 ; ledIndex++) { //sizeof(ledState)/sizeof(uint32_t)
    
    int indexDiv = ledIndex / 3 ;
    
    tlc.setLED(indexDiv, ledState[indexDiv*3], ledState[indexDiv*3+1], ledState[indexDiv*3+2]);
//    Serial.print(ledState[ledIndex]);
//    Serial.print(" ");
  }
//  Serial.println();
  tlc.write();
}

void buttonFocus(int index) {
  currentButton = index ;
  Serial.println(currentButton);
  //turn off all leds but currentButton one
  for (int i ; i < sizeof(ledState)/sizeof(uint32_t) ; i++) {

    if (i == currentButton ) { 
      ledState[11-i] = 65535 ;
    }
    else { ledState[11-i] =  0 ; }
    tlcWrite();
  }
  
  EvtResetButtonContext() ; //clean context before starting reading the file
  
  //et après ya plein de trucs à faire
  String file = "button";
  file+=index ;
  file+=".txt";
  btnFile = SD.open("button0.txt") ;
  Serial.println("opening " + file);
  if (!file) {
    Serial.print("The text file cannot be opened");
//    return false;
   }
   readSDFile();

}

bool button0(){
  buttonFocus(0) ;
  return false;
}
bool button1(){
  buttonFocus(1) ;
  return false;
}
bool button2(){
  buttonFocus(2) ;
  return false;
}
bool button3(){
  buttonFocus(3) ;
  return false;
}
bool button4(){
  buttonFocus(4) ;
  return false;
}
bool button5(){
  buttonFocus(5) ;
  return false;
}
bool button6(){
  buttonFocus(6) ;
  return false;
}
bool button7(){
  buttonFocus(7) ;
  return false;
}
bool button8(){
  buttonFocus(8) ;
  return false;
}
bool button9(){
  buttonFocus(9) ;
  return false;
} 

void EvtResetButtonContext () {
  mgr.resetContext();
  mgr.addListener(new EvtPinListener(BUTTON_PIN[0], (EvtAction)button0));
  mgr.addListener(new EvtPinListener(BUTTON_PIN[1], (EvtAction)button1));
  mgr.addListener(new EvtPinListener(BUTTON_PIN[2], (EvtAction)button2));
  mgr.addListener(new EvtPinListener(BUTTON_PIN[3], (EvtAction)button3));
  mgr.addListener(new EvtPinListener(BUTTON_PIN[4], (EvtAction)button4));
  mgr.addListener(new EvtPinListener(BUTTON_PIN[5], (EvtAction)button5));
  mgr.addListener(new EvtPinListener(BUTTON_PIN[6], (EvtAction)button6));
  mgr.addListener(new EvtPinListener(BUTTON_PIN[7], (EvtAction)button7));
  mgr.addListener(new EvtPinListener(BUTTON_PIN[8], (EvtAction)button8));
  mgr.addListener(new EvtPinListener(BUTTON_PIN[9], (EvtAction)button9));
}






///////////////////////////////////////////////////////////////////////

USE_EVENTUALLY_LOOP(mgr) // Use this instead of your loop() function.
